# ifndef SSLBUFFER_HPP
# define SSLBUFFER_HPP

#include	<dvutil/fdstreambuf.h>
#include	<dvssl/sslcontext.h>

/*! \file
    This file defined Dv::Ssl::SslBuffer, which is derived from
    Dv::Ssl::fdstreambuf.
*/
namespace Dv {
namespace Ssl {

//! A streambuf class for use with Dv::Ssl::SslSocket iostream objects.
class SslBuffer: public Dv::Util::fdstreambuf {
public:
  //! Constructor. Msecs is max timeout value in millisecs.
  SslBuffer(SslContext& context,int fd,size_t intbufsize=1024,
	    size_t outbufsize=1024, int msecs = 0);
  //! Destructor.
  ~SslBuffer();

  //! Return pointer to SSL object. 
  /*! Return type is declared as void* to avoid inclusion of openssl 
      header files.
  */
  void*		ssl() const	{ return ssl_; }
  //! Return string representation of cypher.
  const char*	cipher() const;
protected:
  //! Raw read implementation, see Dv::Util::fdstreambuf.
  virtual int	rread(char* buf, size_t len);
  //! Raw write implementation, see Dv::Util::fdstreambuf.
  virtual int	rwrite(char* buf, size_t len);
private:
  //! Opaque, really a SSL* pointer.
  void		*ssl_; // opaque, really a SSL*
};

}}
#endif
