/// $Id: sslcontext.C,v 1.4 2003/06/22 12:18:45 dvermeir Exp $

#include <stdexcept>
#include <iostream>
#include <openssl/ssl.h>

#include "sslcontext.h"
#include "sslinit.h"

namespace Dv {
namespace Ssl {
///////////////	Init
class Init {
public:
  Init() throw (std::runtime_error);
  ~Init();
};
}}


Dv::Ssl::Init::Init() throw (std::runtime_error) {
// Initialize openssl
if (!dv_ssl_setup())
  throw std::runtime_error("Cannot initialize Dv::Ssl");
SSL_load_error_strings();
SSLeay_add_ssl_algorithms();
}

Dv::Ssl::Init::~Init() {
dv_ssl_cleanup();
}

using namespace std;
namespace Dv {
namespace Ssl {
///////////////	SslContext 

SslContext::SslContext() throw (std::runtime_error): context_(0), 
  rsa_private_key_file_(0), certificate_file_(0) {
try {
  static Dv::Ssl::Init init;
  }
catch (std::exception& e) {
  std::cerr << e.what() << std::endl;
  throw;
  }
}

SslContext::~SslContext() {
if (context_)
  SSL_CTX_free(static_cast<SSL_CTX*>(context_));
delete rsa_private_key_file_;
delete certificate_file_;
}

bool
SslContext::rsa_private_key_file(const char* filename) {
if (!filename)
  return false;
rsa_private_key_file_ = new string(filename);
if (!context_) 
  return false;
return (SSL_CTX_use_RSAPrivateKey_file(
  static_cast<SSL_CTX*>(context_), filename, SSL_FILETYPE_PEM) == 1);
}

bool
SslContext::certificate_file(const char* filename) {
if (!filename)
  return false;
if (!context_) 
  return false;
certificate_file_ = new string(filename);
return (SSL_CTX_use_certificate_file(
 static_cast<SSL_CTX*>(context_),filename, SSL_FILETYPE_PEM)==1);
}

///////////////	SslContextV2

SslContextV2::SslContextV2(const char* keyfile, const char* certfile)
 throw (runtime_error): SslContext() {
static const string ERR("SslContextV2::SslContextV2(): ");

context_ = SSL_CTX_new(SSLv2_method());
if (context_ == 0)
  throw runtime_error(ERR+"SSL_CTX_new(SSLv2_method()) fails");

// The following line is useless.
SSL_CTX_set_verify(static_cast<SSL_CTX*>(context_),SSL_VERIFY_NONE,0);
if (keyfile) 
  if (! rsa_private_key_file(keyfile))
    throw runtime_error(ERR+"cannot use key file \""+keyfile+"\"");
if (certfile) 
   if (!certificate_file(certfile))
      throw runtime_error(ERR+"cannot use cert file \""+certfile+"\"");
}

SslContextV2::~SslContextV2() {
}

///////////////	SslContextV23

SslContextV23::SslContextV23(const char* keyfile, const char* certfile)
  throw (runtime_error): SslContext() {
static const string ERR("SslContextV3::SslContextV3(): ");
context_ = SSL_CTX_new(SSLv23_method());
if (context_ == 0)
  throw runtime_error(ERR+"SSL_CTX_new(SSLv2_method()) fails");
SSL_CTX_set_verify(static_cast<SSL_CTX*>(context_),SSL_VERIFY_NONE,0);
if (keyfile) 
  if (! rsa_private_key_file(keyfile))
    throw runtime_error(ERR+"cannot use key file \""+keyfile+"\"");
if (certfile) 
   if (!certificate_file(certfile))
      throw runtime_error(ERR+"cannot use cert file \""+certfile+"\"");
}

SslContextV23::~SslContextV23() {
}

///////////////	SslContextV3
 
SslContextV3::SslContextV3(const char* keyfile, const char* certfile)
  throw (runtime_error): SslContext() {
static const string ERR("SslContextV23::SslContextV23(): ");
context_ = SSL_CTX_new(SSLv3_method());
if (context_ == 0)
  throw runtime_error(ERR+"SSL_CTX_new(SSLv2_method()) fails");
SSL_CTX_set_verify(static_cast<SSL_CTX*>(context_),SSL_VERIFY_NONE,0);
if (keyfile) 
  if (! rsa_private_key_file(keyfile))
    throw runtime_error(ERR+"cannot use key file \""+keyfile+"\"");
if (certfile) 
   if (!certificate_file(certfile))
      throw runtime_error(ERR+"cannot use cert file \""+certfile+"\"");
}

SslContextV3::~SslContextV3() {
}

}}
 
