// $Id: sslinit.c,v 1.3 2003/07/23 16:11:45 dvermeir Exp $
#include <pthread.h>
#include <stdio.h>
#include <openssl/ssl.h>
#include <openssl/rand.h>

static pthread_mutex_t* mutex_buf = 0;

void
dv_ssl_locking_function(int mode, int n, const char* file, int line) {
if (mode & CRYPTO_LOCK)
  pthread_mutex_lock(&(mutex_buf[n]));
else
  pthread_mutex_unlock(&(mutex_buf[n]));
}

static int
dv_ssl_thread_setup(void) {
int i;
mutex_buf = (pthread_mutex_t*) malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t) );
if (!mutex_buf)
  return 0;
for (i=0; i < CRYPTO_num_locks(); i++)
  pthread_mutex_init(&(mutex_buf[i]), NULL);
CRYPTO_set_id_callback(pthread_self);
CRYPTO_set_locking_callback(dv_ssl_locking_function);
return 1;
}

static int
dv_ssl_thread_cleanup(void) {
int i;
if (!mutex_buf)
  return 0;
CRYPTO_set_id_callback(NULL);
CRYPTO_set_locking_callback(NULL);
for (i=0; i < CRYPTO_num_locks(); i++)
  pthread_mutex_destroy(&(mutex_buf[i]));
free(mutex_buf);
mutex_buf = NULL;
return 1;
}

int
dv_ssl_cleanup(void) {
// fprintf(stderr, "dv_ssl_cleanup\n");
return dv_ssl_thread_cleanup();
}

int
dv_ssl_setup(void) {
// fprintf(stderr, "dv_ssl_setup\n");
// We use urandom because using random sometime hangs (when the
// system thinks it has not enough entropy, presumably).
if (!RAND_load_file("/dev/urandom", 1024))
  return 0;
return dv_ssl_thread_setup();
}

