#ifndef	SSLSERVERSOCKET_H
#define	SSLSERVERSOCKET_H

// $Id: sslserversocket.h,v 1.5 2002/10/19 10:36:49 dvermeir Exp $

#include	<stdexcept>
#include	<dvssl/sslcontext.h>
#include	<dvssl/sslsocket.h>
#include	<dvnet/serversocket.h>
/* \file
   This file defines the Dv::Ssl::SslServerSocket class.
*/

namespace Dv {
namespace Ssl {

//! A class derived from Dv::Ssl::ServerSocket that uses SSL.
/*! Example usage:
    \code
    try {
      SslContextV23	context("key.pem","cert.pem");
      SslServerSocket	server(context,port);

      cerr << "SSL echo server started.." << endl;
      ref<SslSocket>	client(server.accept());

      string line;
      while (getline(*client,line))
        *client << line << endl;
      cout << "client status: " << client->strerror() << endl;
      return server.error();
      }
    catch (exception& e) {
      cerr << e.what() << endl;
      return 2;
      }
    \endcode
*/
class SslServerSocket: public Net::ServerSocket {
public:

  //! Constructor.
  /*! The Dv::Ssl::SslContext parameter must contain both a 
      key file and a certificate file.
  */
  SslServerSocket(SslContext& context,int port,int backlog = 10) 
  						throw (std::runtime_error);
  //! Accept a connection from an SSL client.
  /*! Delay and bufsz are used for the resulting SslSocket.
      \param delay number of millisecs resulting socket will wait for I/O
      \param bufsz buffer size of streambuf associated with resulting socket

      Note that this is different from the use of the <code>delay</code>
      parameter for the userversocket::accept() function.
  */
  ref<SslSocket>	accept(time_t delay = 0, size_t bufsz=1024 );
private:
  SslContext&	context_;
};

}}
#endif

