#ifndef  SSLSOCKET_H
#define  SSLSOCKET_H
// $Id: sslsocket.h,v 1.5 2002/10/19 10:36:49 dvermeir Exp $

#include  <dvssl/sslbuffer.h>
#include  <dvnet/socket.h>
/*! \file
    This file declares the Dv::Ssl::SslSocket class.
*/
namespace Dv {
namespace Ssl {

using Dv::Util::ref;

class SslServerSocket;

//! An iostream derived from Dv::Net::Socket.
/*! Example usage:
    \code
    SslContextV23	context;
    SslSocket	        client(context,"host.domain",9999);

    if (!client) {
      cerr	<< "connection failed: " << client.strerror() << endl;
      return 1;
      }

    X509Certificate cert(client);
    cout << "client: certificate name = " << cert.name() << endl
         << "client: certificate issuer = " << cert.issuer() << endl;

    const string out("hello world");
    client << out << endl;
    
    string line;
    getline(client,line);
    if (line!=out) {
      cerr << "Client expected \"" << out << "\", got \"" << line << "\"" << endl;
      return 1;
      }

   cout << "Client exit status:" << client.strerror() << endl;
   return client.error();
   \endcode
*/
class SslSocket: public Net::Socket {

/** SslServerSocket is a friend. */
friend class SslServerSocket;

public:
  //! Extra error code (see Dv::Net::Socket) for SSL connections.
  enum { SSL_CONNECTION_FAILED = -30, SSL_ACCEPT_FAILED = -31 };

  //! Constructor.
  /*! The Dv::Ssl::SslContext parameter need not contain private key or
      certificate file information since it is not used.
  */
  SslSocket::SslSocket(SslContext& context,const std::string& host,int port,
  	size_t bufsize = 1024, int msecs=0);
  //! Destructor.
  ~SslSocket();

  //! Return SSL* pointer, opaque to avoid inclusion of openssl header files.
  void*		ssl() const; 
  //! Return string representation of used cipher.
  const char*	cipher() const;
  //! Overrides Dv::Net::Socket::strerror.
  std::string	strerror() const;

private:
  //! Constructor version used by Dv::Ssl::SslSocket::fs2socket.
  SslSocket::SslSocket(SslContext& context,int fd,size_t bufsize=1024, int msecs = 0);
  //! Used internally by Dv::Ssl::SslServerSocket::accept.
  static ref<SslSocket>	fd2sslsocket(SslContext&, int fd, size_t bufsz=1024, int msecs=0);

  //! Associated SSL context.
  SslContext&	context_;
  //! Associated streambuf.
  SslBuffer*	sslbuf_;
};

}}
#endif
