// $Id: test-echocs.C.in,v 1.2 2002/10/19 10:43:49 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>
#include <dvssl/sslsocket.h>
#include <dvssl/x509cert.h>
#include <dvssl/sslserversocket.h>

using namespace std;
using namespace Dv;
using namespace Ssl;

int // return 0 iff ok
echo_client(const string& host,int port) {
cerr << "Client connecting to " << host << ":" << port << endl;

try {
  SslContextV23	context;
  SslSocket	client(context,host,port);

  if (!client) {
    cerr	<< "connection failed: " << client.strerror() << endl;
    return 1;
    }

  X509Certificate cert(client);
  cerr << "client: certificate name = " << cert.name() << endl
       << "client: certificate issuer = " << cert.issuer() << endl;

  ifstream ifs("/etc/passwd");
  if (!ifs) {
    cerr << "Cannot open /etc/passwd" << endl;
    return 1;
    }

  string line;
  string ifsline;
  while (getline(ifs,ifsline)) {
    client << ifsline << endl;
    getline(client,line);
    if (line!=ifsline) {
      cerr << "Client expected \"" << ifsline 
           << "\", got \"" << line << "\"" << endl;
      return 1;
      }
    }
  cerr << "Client exit:" << client.strerror() << endl;
  return client.error();
  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 2;
  }
}

int
main(int argc,char* argv[]) {
const char* const usage = "techo";

int port(9999);

// The parent will be the server, the child the client.

int child = fork();

if (child<0) {
  cerr << "fork() failed" << endl;
  return 1;
  }
  
if (child==0) {
  sleep(3);
  cerr << "Starting client.." << endl;
  return echo_client("localhost",port);
  }

try {
  SslContextV23	context("./key.pem","./cert.pem");
  SslServerSocket	server(context,port);

  cerr << "Starting server.." << endl;

  ref<SslSocket>	client(server.accept());

  string line;
  while (getline(*client,line))
    *client << line << endl;
  cerr << "client: " << client->strerror() << endl;
  return 0;
  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 2;
  }

int client_status;
wait(&client_status);

if (WIFEXITED(client_status)) 
   if (WEXITSTATUS(client_status)==0)
     return 0;
   else
     return 1;
else
  return 1;
}
