// $Id: test-mt-echocs.C,v 1.2 2003/07/23 16:19:52 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>
#include <dvutil/systemerror.h>
#include <dvssl/sslsocket.h>
#include <dvssl/x509cert.h>
#include <dvssl/sslserversocket.h>


typedef Dv::Util::ref<Dv::Ssl::SslSocket> REF_SO;

void*
server_thread(void* void_client_ptr) {
REF_SO* client_ptr(reinterpret_cast<REF_SO*>(void_client_ptr));
REF_SO client(*client_ptr);
delete client_ptr; // was new'ed in launch_handler
std::cerr << pthread_self() << ": started" << std::endl;
pthread_detach(pthread_self());
std::string line;
while (getline(*client,line))
  *client << line << std::endl;
std::cerr << pthread_self() << ": " << client->strerror() << std::endl;
pthread_exit(0);
return 0;
}

pthread_t
launch_handler(REF_SO client) {
pthread_t tid;
// This is necessary since otherwise the server main thread may
// have removed the local client (and the main one) by the time
// the handler thread starts.
REF_SO* client_ptr = new REF_SO(client); // deleted in server_thread()
if (pthread_create( &tid, 0, server_thread, client_ptr)) 
  throw Dv::Util::SystemError();
return tid;
}

int // return 0 iff ok
echo_client(const std::string& host, int port) {
std::cerr << "client: connecting to " << host << ":" << port << std::endl;
try {
  Dv::Ssl::SslContextV23 context;
  Dv::Ssl::SslSocket client(context, host, port);

  if (!client) {
    std::cerr << "client: connection failed: " << client.strerror() << std::endl;
    return 1;
    }

  Dv::Ssl::X509Certificate cert(client);
  std::cerr << "client: certificate name = " << cert.name() << std::endl
       << "client: certificate issuer = " << cert.issuer() << std::endl;

  std::ifstream ifs("/etc/passwd");
  if (!ifs) {
    std::cerr << "client: cannot open /etc/passwd" << std::endl;
    return 1;
    }

  std::string line;
  std::string ifsline;
  while (std::getline(ifs, ifsline)) {
    std::cerr << "client: sending " << ifsline << std::endl;
    client << ifsline << std::endl;
    std::getline(client, line);
    if (line != ifsline) {
      std::cerr << "client: expected \"" << ifsline 
           << "\", got \"" << line << "\"" << std::endl;
      return 1;
      }
    }
  std::cerr << "client: exit: " << client.error() << ": " << client.strerror() << std::endl;
  return client.error();
  }
catch (std::exception& e) {
  std::cerr << "client: exception: " << e.what() << std::endl;
  return 2;
  }
return 0;
}

int // return 0 iff ok
echo_server(int port) {
try {
  Dv::Ssl::SslContextV23 context(SRCDIR "/key.pem",SRCDIR "/cert.pem");
  Dv::Ssl::SslServerSocket server(context, port);

  std::cerr << "server: start" << std::endl;

  Dv::Util::ref<Dv::Ssl::SslSocket> client(server.accept());
  pthread_t tid = launch_handler(client);
  std::cerr << "server: joining thread " << tid << std::endl;
  return pthread_join(tid, 0);
  }
catch (std::exception& e) {
  std::cerr << "server: " << e.what() << std::endl;
  return 2;
  }
}

int
main(int argc, char* argv[]) {
// const char* const usage = "techo-mt";

int port(9999);
// The parent will be the server, the child the client.

int child = fork();

if (child<0) {
  std::cerr << "fork() failed" << std::endl;
  return 1;
  }

if (child==0) {
  // Child
  sleep(3);
  std::cerr << "client: start" << std::endl;
  int status = echo_client("localhost", port);
  std::cerr << "client: exit with status = " << status << std::endl;
  exit(status);
  }
else {
  int server_status = echo_server(port);
  std::cerr << "main: server_status = " << server_status << std::endl;
  int client_status(1); // default: trouble

  std::cerr << "main: waiting for child to die" << std::endl;

  int child_exit_status; 
  wait(&child_exit_status);
  std::cerr << "main: child_exit_status = " << child_exit_status << std::endl;

  if (WIFEXITED(child_exit_status)) {
     client_status = WEXITSTATUS(child_exit_status);
     std::cerr << "main: client client_status = " << child_exit_status << std::endl;
     }
  else {
    if (WIFSIGNALED(child_exit_status)) {
      std::cerr << "main: WIFSIGNALED: signal = " << WTERMSIG(child_exit_status) << std::endl;
      client_status = 1;
      }
    else {
      std::cerr << "server: WIFEXITED returns strange stuff" << std::endl;
      client_status = 2;
      }
    }
  return (100 * server_status + client_status);
}
}
